// IRMon
// irmon@lucaseckels.com

// Really simple Arduino IR remote listener.
// Uses a simple loop to time the pulses that come out of a 38 kHz
// IR decoder.  It learns the first message, than looks for the same
// set later.

#define POWER_PIN 7
#define GND_PIN 6
#define DATA_PIN 5
#define BLINK_PIN 13

#define ARRAY_SIZE 64
#define FUZZ 30
#define MAX_TIME 1000

void setup() {
  pinMode(POWER_PIN, OUTPUT);
  pinMode(GND_PIN, OUTPUT);
  pinMode(DATA_PIN, INPUT);
  pinMode(BLINK_PIN, OUTPUT);
  
  digitalWrite(POWER_PIN, HIGH);
  digitalWrite(GND_PIN, LOW);
  digitalWrite(BLINK_PIN, LOW);
  
  Serial.begin(9600);

  delay(500);
}

// Write out over serial the values
void outputValues(int *values, int count) {
  if (count == 0) {
    Serial.print("Empty\n");
  }
  for (int i = 0; i < count; ++i) {
   if (i % 2) {
     Serial.print("On: ");
   } else {
     Serial.print("Off: ");
   }
   Serial.print(values[i]);
   Serial.print("\n");  
  }
}

bool compare(int *values1, int *values2, int count) {
  for (int i = 0; i < count; ++i) {
    if (abs(values1[i] - values2[i]) > FUZZ) {
      return false;
    }
  }
  return true;
}

void loop() {
  int learnedValues[ARRAY_SIZE];
  int count = measure(learnedValues);
  outputValues(learnedValues, count);
  
  while (true) {
    int readValues[ARRAY_SIZE];
    int readCount = measure(readValues);
    if (readCount == count && compare(learnedValues, readValues, count)) {
      Serial.print("Recognized\n");
      digitalWrite(BLINK_PIN, HIGH);
      delay(500);
      digitalWrite(BLINK_PIN, LOW);
    } else {
      Serial.print("Not recognized\n");
    }
  }
}

int measure(int *values) {
  int count = 0;
  bool last = false;
  memset(values, 0, ARRAY_SIZE);
  
  while (digitalRead(DATA_PIN) == HIGH)
    ;
    
  while (count < ARRAY_SIZE) {
    bool val = digitalRead(DATA_PIN);
    if (val == last) {
      if (++values[count] >= MAX_TIME) {
        return count;
      }
    } else {
      ++count;
      last = val;
    }  
  }
  
  return 0;
}
